<?php

namespace FiloBlu\Refilo\Model\Adapter;

use FiloBlu\Refilo\Helper\Data as HelperData;
use MongoDB\Driver\BulkWrite;
use MongoDB\Driver\Command;
use MongoDB\Driver\Exception\Exception;
use MongoDB\Driver\Manager;

class Mongo extends AbstractAdapter
{
    /**
     * @var HelperData
     */
    private $helperData;

    /**
     * Mongo constructor.
     * @param HelperData $helperData
     */
    public function __construct(
        HelperData $helperData
    ) {
        $this->helperData = $helperData;
    }

    public function connect()
    {
        if (!$this->_connection) {
            $this->_connection = new Manager($this->helperData->getConfigValue('mongo/general/mongo_host'));
        }

        return $this->_connection;
    }

    /**
     * @param $collection
     * @param $mongoDatabase
     * @throws Exception
     */
    public function dropCollection($collection, $mongoDatabase)
    {
        $collectionExist = !empty($this->connect()->executeCommand($mongoDatabase, new Command(['listCollections' => 1, 'filter' => ['name' => $collection]]))->toArray());

        if ($collectionExist) {
            try {
                $this->_connection->executeCommand($mongoDatabase, new Command(["drop" => $collection]));
                if (php_sapi_name() === 'cli') {
                    printf("Dropped %s collection\n", "{$collection}");
                }
            } catch (Exception $e) {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param $collection
     * @param $mongoDatabase
     * @throws Exception
     */
    public function flushCollection($collection, $mongoDatabase)
    {
        $collectionExist = !empty($this->connect()->executeCommand($mongoDatabase, new Command(['listCollections' => 1, 'filter' => ['name' => $collection]]))->toArray());

        if ($collectionExist) {
            try {
                $this->connect()->executeCommand($mongoDatabase, new Command(['drop' => $collection]));
                if (php_sapi_name() === 'cli') {
                    printf("All documents removed from %s collection\n", (string)($collection));
                }
            } catch (Exception $e) {
                echo $e->getMessage();
            }
        } else {
            try {
                $this->connect()->executeCommand($mongoDatabase, new Command(["create" => $collection]));
                if (php_sapi_name() === 'cli') {
                    printf("%s collection regenerated without documents\n", (string)($collection));
                }
            } catch (Exception $e) {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param $key
     * @param $objects
     * @param $collection
     * @param bool $insertMode
     * @param $mongoDatabase
     * @return mixed|void
     */
    public function update($key, $objects, $collection, $insertMode, $mongoDatabase)
    {
        $bulk = new BulkWrite();

        foreach ($objects as $object) {
            $objToUpdate = $object;
            if ($insertMode === false) {
                $objToUpdate = ['$set' => $object];
            }
            $bulk->update(['_id' => $object->{$key}], $objToUpdate, ['multi' => false, 'upsert' => $insertMode]);
        }

        try {
            $result = $this->connect()->executeBulkWrite(sprintf('%s.%s', $mongoDatabase, $collection), $bulk);
            if (php_sapi_name() === 'cli') {
                printf("Inserted %d document(s)\n", $result->getInsertedCount());
                printf("Matched  %d document(s)\n", $result->getMatchedCount());
                printf("Updated  %d document(s)\n", $result->getModifiedCount());
                printf("Upserted %d document(s)\n", $result->getUpsertedCount());
                printf("Deleted  %d document(s)\n", $result->getDeletedCount());
            }
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    /**
     * @param $fields
     * @param $collection
     * @return mixed
     * @throws Exception
     */
    public function ensureIndex($fields, $collection)
    {
        try {
            $database = $this->helperData->getConfigValue('mongo/general/mongo_general_database');
            $indexes = [];

            foreach ($fields as $field) {
                $indexes[] = [
                    'name' => $field,
                    'key' => [$field => 1],
                    'ns' => "{$database}.{$collection}",
                ];
            }

            $command = new Command([
                'createIndexes' => $collection,
                'indexes' => $indexes,
            ]);

            $this->connect()->executeCommand($database, $command);
            if (php_sapi_name() === 'cli') {
                printf("added Indexes for %s \n", "{$database}.{$collection}");
            }
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }
}
