<?php

namespace FiloBlu\Refilo\Remote;

use Exception;
use FiloBlu\Refilo\Remote\Entity\EntityProviderInterfaceFactory;
use FiloBlu\Refilo\Remote\Helper\TemplateString;
use FiloBlu\Refilo\Remote\Helper\TemplateStringFactory;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerConfigurationInterface;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerConfigurationRepositoryInterface;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerResolver;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Psr\Log\LoggerInterface;

/**
 * Class AbstractIndexer
 * @package FiloBlu\Refilo\Remote
 */
abstract class AbstractIndexer implements IndexerInterface
{
    /**
     * @var EntityProviderInterfaceFactory
     */
    protected $entityProviderFactory;
    /**
     * @var string
     */
    private $name;
    /**
     * @var IndexerConfigurationRepositoryInterface
     */
    private $indexerConfigurationRepository;
    /**
     * @var IndexerConfigurationInterface
     */
    private $indexerConfiguration;
    /**
     * @var TemplateStringFactory
     */
    private $templateStringFactory;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;
    /**
     * @var  TemplateString
     */
    private $remoteIndexTemplate;
    /**
     * @var LoggerInterface
     */
    private $logger;
    /**
     * @var IndexerConfiguration\IndexerResolver
     */
    private $indexerResolver;

    /**
     * AbstractIndexer constructor.
     * @param EntityProviderInterfaceFactory $entityProviderFactory
     * @param IndexerConfigurationRepositoryInterface $indexerConfigurationRepository
     * @param TemplateStringFactory $templateStringFactory
     * @param IndexerConfiguration\IndexerResolver $indexerResolver
     * @param ScopeConfigInterface $scopeConfig
     * @param LoggerInterface $logger
     */
    public function __construct(
        EntityProviderInterfaceFactory $entityProviderFactory,
        IndexerConfigurationRepositoryInterface $indexerConfigurationRepository,
        TemplateStringFactory $templateStringFactory,
        IndexerResolver $indexerResolver,
        ScopeConfigInterface $scopeConfig,
        LoggerInterface $logger
    ) {
        $this->entityProviderFactory = $entityProviderFactory;
        $this->indexerConfigurationRepository = $indexerConfigurationRepository;
        $this->templateStringFactory = $templateStringFactory;
        $this->scopeConfig = $scopeConfig;
        $this->logger = $logger;
        $this->indexerResolver = $indexerResolver;
    }

    /**
     * @param array $ids
     * @return mixed
     */
    abstract public function getEntitiesToRemove(array $ids = []);

    /**
     *
     */
    public function executeFull()
    {
        $this->executeList([]);
    }

    /**
     * @param array $ids
     */
    public function executeList(array $ids)
    {
        try {
            $ids = array_unique($ids);
            $this->beforeExecute($ids);
            $this->executeAction($ids);
            $this->postExecute($ids);
        } catch (Exception $exception) {
            $this->logger->error($exception);
        }
    }

    /**
     * @param array $ids
     */
    abstract public function beforeExecute(array $ids = []);

    /**
     * @param array $ids
     * @return mixed
     */
    abstract public function executeAction(array $ids);

    /**
     * @param array $ids
     */
    abstract public function postExecute(array $ids = []);

    /**
     * @param int $id
     * @throws NoSuchEntityException
     */
    public function executeRow($id)
    {
        $this->executeList([$id]);
    }

    /**
     * Invoked by indexer_update_all_views
     *
     * @param int[] $ids
     */
    public function execute($ids)
    {
        $this->executeList([$ids]);
    }

    /**
     * @return TemplateString
     * @throws NoSuchEntityException
     */
    public function getGetRemoteIndexTemplate(): TemplateString
    {
        if ($this->remoteIndexTemplate) {
            return $this->remoteIndexTemplate;
        }

        $projectId = strtolower($this->scopeConfig->getValue('filoblu/general/project_id'));

        /** @var TemplateString $indexTemplate */
        $this->remoteIndexTemplate = $this->templateStringFactory->create();

        $this->remoteIndexTemplate
            ->withString($this->getIndexerConfiguration()->getIndexName())
            ->withValue('%p', $projectId);

        return $this->remoteIndexTemplate;
    }

    /**
     * @return IndexerConfigurationInterface
     * @throws NoSuchEntityException
     */
    public function getIndexerConfiguration(): IndexerConfigurationInterface
    {
        if ($this->indexerConfiguration === null) {
            /** @var IndexerConfigurationInterface indexerConfiguration */
            $this->indexerConfiguration = $this->indexerConfigurationRepository->getByIndexerId($this->getName());
        }

        return $this->indexerConfiguration;
    }

    /**
     * @return string
     * @throws NoSuchEntityException
     */
    protected function getName()
    {
        if ($this->name === null) {
            $this->name = $this->indexerResolver->resolve($this->getClass());
        }

        return $this->name;
    }

    /**
     * @return string
     */
    abstract protected function getClass(): string;

    /**
     * @return Connector\ConnectorInterface
     * @throws NoSuchEntityException
     */
    public function getConnector(): Connector\ConnectorInterface
    {
        return $this->getIndexerConfiguration()->getConnectorConfiguration()->getConnector();
    }

    /**
     * @return LoggerInterface
     */
    protected function getLogger()
    {
        return $this->logger;
    }

    /**
     * @param $entities
     * @param $collection
     * @return mixed
     */
    abstract protected function write($entities, $collection);
}
