<?php

namespace FiloBlu\Refilo\Remote\Entity;

use FiloBlu\Refilo\Remote\Entity\Provider\ProductAttributeProvider;
use Magento\Catalog\Api\Data\ProductInterface;

/**
 * Class MagentoToRefiloProductAdapter
 * @package FiloBlu\Refilo\Remote\Entity
 */
class MagentoToRefiloProductAdapter
{
    /**
     * @var ProductAttributeProvider
     */
    private $productAttributeProvider;

    /**
     * MagentoToRefiloProductAdapter constructor.
     * @param ProductAttributeProvider $productAttributeProvider
     */
    public function __construct(ProductAttributeProvider $productAttributeProvider)
    {
        $this->productAttributeProvider = $productAttributeProvider;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    public function adapt(ProductInterface $magentoProduct, Product $refiloProduct): Product
    {
        $refiloProduct->setData(
            [
                Product::ID               => (int)$magentoProduct->getEntityId(),
                Product::SKU              => $magentoProduct->getSku(),
                Product::TYPE             => $magentoProduct->getTypeId(),
                Product::NAME             => $magentoProduct->getName(),
                Product::CREATED_AT       => $magentoProduct->getCreatedAt(),
                Product::UPDATED_AT       => $magentoProduct->getUpdatedAt(),
                Product::WEIGHT           => $magentoProduct->getWeight(),
                Product::META_TITLE       => $magentoProduct->getMetaTitle(),
                Product::META_KEYWORDS    => $magentoProduct->getMetaKeywords(),
                Product::META_DESCRIPTION => $magentoProduct->getMetaDescription(),
                Product::VISIBLE          => ((int)$magentoProduct->getVisibility() === 1 ? '0' : '1'),
                Product::CATEGORIES       => array_map('intval', $magentoProduct->getCategoryIds()),
                Product::ATTRIBUTES       => $this->productAttributeProvider->getAttributesValues($magentoProduct),
                Product::FILTERS          => $this->productAttributeProvider->getFilterableAttributes($magentoProduct),
                Product::SORTABLES        => $this->productAttributeProvider->getSortableAttributes($magentoProduct),
                Product::UPSELL           => $magentoProduct->getUpSellProductIds(),
                Product::CROSSSELL        => $magentoProduct->getCrossSellProductIds(),
                Product::RELATED          => $magentoProduct->getRelatedProductIds(),
                Product::ASSETS           => $this->getAssets($magentoProduct)
            ]
        );

        $this->fillChildren($magentoProduct, $refiloProduct);
        return $refiloProduct;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @return array
     */
    protected function getAssets(ProductInterface $magentoProduct): array
    {
        $assets = [];
        $gallery = $magentoProduct->getData('media_gallery');

        foreach ($gallery['images'] as $image) {
            $assets['images'][] = [
                'label' => $image['label'] ?? null, // TODO : remove?
                'alt'   => $image['label'] ?? null,
                'file'  => ($image['file']) ? "/catalog/product{$image['file']}" : null
            ];
        }

        $assets['placeholders'] = $this->productAttributeProvider->getImagePlaceholdersAttributes($magentoProduct);

        return $assets;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    protected function fillChildren(ProductInterface $magentoProduct, Product $refiloProduct)
    {
        switch ($magentoProduct->getTypeId()) {
            case 'configurable':
                return $this->getConfigurableChildren($magentoProduct, $refiloProduct);
            case 'bundle':
                return $this->getBundleChildren($magentoProduct, $refiloProduct);
            case 'grouped':
                return $this->getGroupedChildren($magentoProduct, $refiloProduct);
        }

        return $refiloProduct;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    public function getConfigurableChildren(ProductInterface $magentoProduct, Product $refiloProduct): Product
    {
        if ($magentoProduct->getTypeId() === 'configurable') {

            $extensionAttributes = $magentoProduct->getExtensionAttributes();

            $refiloProduct->setChildren(array_map('intval', array_values($extensionAttributes->getConfigurableProductLinks())));
            $productOptions = $extensionAttributes->getConfigurableProductOptions() ?? [];

            $variants = [];
            foreach ($productOptions as $option) {
                $attributeCode = $option->getProductAttribute()->getAttributeCode();
                $variant = [
                    'id'             => $option->getId(),
                    'type'           => $option->getProductAttribute()->getData('frontend_input'),
                    'position'       => $option->getPosition(),
                    'label'          => $option->getProductAttribute()->getStoreLabel(),
                    'use_default'    => (int)$option->getIsUseDefault(),
                    'attribute_id'   => $option->getAttributeId(),
                    'attribute_code' => $attributeCode,
                    'values'         => $option->getOptions(),
                    'options'        => []
                ];

                $additionalData = json_decode($option->getProductAttribute()->getData('additional_data'), true);

                if ($additionalData) {
                    $variant['rendered'] = isset($additionalData['swatch_input_type']) ? $additionalData['swatch_input_type'] : '';
                }

                $labels = [];
                $options = [];
                foreach ($variant['values'] as $value) {
                    $options[] = (int)$value['value_index'];
                    $labels[] = $value['store_label'];
                }

                $variant['options'] = $options;
                $refiloProduct->addFilter($attributeCode, [
                    'title' => $variant['label'],
                    'code'  => $attributeCode,
                    'type'  => $option->getFontendInput(),
                    'key'   => $options,
                    'value' => $labels
                ]);
                $variants[] = $variant;
            }

            $refiloProduct->setVariants($variants);
        }

        return $refiloProduct;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    public function getBundleChildren(ProductInterface $magentoProduct, Product $refiloProduct)
    {
        $extensionAttributes = $magentoProduct->getExtensionAttributes();
        $options = [];

        $refiloProduct->setData('bundle_price_type', (int)$magentoProduct->getPriceType() == 0 ? 'dynamic' : 'fixed');
        foreach ($extensionAttributes->getBundleProductOptions() as $bundleProductOption) {
            $productLinks = $bundleProductOption->getData('product_links');

            if (empty($productLinks)) {
                continue;
            }

            $option = [
                'id'       => (int)$bundleProductOption->getData('option_id'),
                'title'    => $bundleProductOption->getData('title'),
                'required' => (int)$bundleProductOption->getData('required'),
                'type'     => $bundleProductOption->getData('type'),
                'position' => (int)$bundleProductOption->getData('position'),
                'childs'   => []
            ];

            foreach ($productLinks as $productLink) {

                $link_data = $productLink->getData();
                $link_data['product_id'] = $link_data['entity_id'];
                $link_data['product_sku'] = $link_data['sku'];

                unset($link_data['entity_id']);
                unset($link_data['sku']);
                unset($link_data['option_id']);

                $child = [];
                foreach ($link_data as $key => $value) {
                    $child[$key] = (is_numeric($value)) ? $value + 0 : $value;
                }

                $option['childs'][] = $child;
            }

            $options[] = $option;
        }

        $refiloProduct->setBundle($options);

        return $refiloProduct;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    public function getGroupedChildren(ProductInterface $magentoProduct, Product $refiloProduct)
    {
        $links = $magentoProduct->getTypeInstance()->getChildrenIds($magentoProduct->getId());

        if (empty($links)) {
            return $refiloProduct;
        }

        $refiloProduct->setChildren(array_values(array_shift($links)));
        return $refiloProduct;
    }
}
